/**
 * Copyright Marcus Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of LibDesQ (https://gitlab.com/DesQ/libdesq)
 * This library contains various  Core and Gui classes which are used
 * through out the DesQ Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#include "IpcClient.hpp"

#include <QLocalSocket>

const int TIMEOUT = 50;

DesQ::IPC::Client::Client( ConnectionMode mode, QObject *parent ) : QLocalSocket( parent ) {
    mCommMode = mode;

    connect(
        this, &QLocalSocket::disconnected, [ = ]() mutable {
            mConnected = false;
            qDebug() << "=> Disconnected!!";
        }
    );
}


void DesQ::IPC::Client::connectToServer( const QString& serverName, QIODevice::OpenMode ) {
    QLocalSocket::connectToServer( serverName, QIODevice::ReadWrite );

    /* Wait at most @TIMEOUT ms - Our connection is faster */
    if ( QLocalSocket::waitForConnected( TIMEOUT ) ) {
        mConnected = true;
        QByteArray regMsg;
        switch ( mCommMode ) {
            case Broadcast: {
                /* This is the last message we will be sending */
                regMsg = "register bcast";
                break;
            }

            case Exchange: {
                /* A long communication will follow */
                regMsg = "register chat";
                break;
            }

            case Message: {
                /* We will be sending just one more command and disconnect */
                regMsg = "register msg";
                break;
            }

            case Generic: {
                regMsg = "register generic";
                break;
            }
        }

        write( regMsg );
        flush();
    }

    else {
        qDebug() << error() << errorString();
    }
}


bool DesQ::IPC::Client::sendMessage( QString msg ) {
    /* Generic mode will always be connected. */
    if ( not mConnected and mCommMode != Generic ) {
        return false;
    }

    this->write( msg.toUtf8() );

    return waitForAck();
}


bool DesQ::IPC::Client::waitForAck( int timeout ) {
    /* Convoluted way to set the default */
    if ( timeout == -1 ) {
        timeout = TIMEOUT;
    }

    /* Convoluted way to set infinite timeout */
    else if ( timeout == 0 ) {
        timeout = -1;
    }

    /* Wait for @TIMEOUT ms for ack */
    if ( waitForReadyRead( timeout ) ) {
        QByteArray ackStr = QLocalSocket::readAll();

        if ( ackStr.size() ) {
            if ( mCommMode == Broadcast ) {
                connect(
                    this, &QLocalSocket::readyRead, [ = ]() {
                        qDebug() << "BCAST:" << QLocalSocket::readAll();
                    }
                );
            }

            return true;
        }
    }

    return false;
}


QByteArray DesQ::IPC::Client::readAll() {
    /* Generic mode will always be connected. */
    if ( not mConnected and mCommMode != Generic ) {
        return QByteArray();
    }

    /* Wait for @TIMEOUT ms for data to be avaialble */
    waitForReadyRead( TIMEOUT );

    /* Data received, read and return it. */
    return QLocalSocket::readAll();
}


void DesQ::IPC::Client::write( QByteArray data ) {
    if ( QLocalSocket::write( data ) < 0 ) {
        disconnectFromServer();
    }

    QLocalSocket::flush();
}


void DesQ::IPC::Client::ack( QByteArray ackMsg ) {
    this->write( "ack " + ackMsg );
}
